Package: Cyclops
Type: Package
Title: Cyclic Coordinate Descent for Logistic, Poisson and Survival
        Analysis
Version: 3.7.1
Authors@R: c(
    person("Marc A.", "Suchard", email = "msuchard@ucla.edu", role = c("aut","cre")),
    person("Martijn J.", "Schuemie", role = "aut"),
    person("Trevor R.", "Shaddox", role = "aut"),
    person("Yuxi", "Tian", role = "aut"),
    person("Jianxiao", "Yang", role = "aut"),
    person("Eric", "Kawaguchi", role = "aut"),
    person("Sushil", "Mittal", role ="ctb"),
    person("Observational Health Data Sciences and Informatics", role = "cph"),
    person("Marcus", "Geelnard", role = c("cph", "ctb"),
      comment = "provided the TinyThread library"),
    person("Rutgers University", role = c("cph", "ctb"),
      comment = "provided the HParSearch routine"),
    person("R Development Core Team", role = c("cph", "ctb"),
      comment = "provided the ZeroIn routine")
    )
Description: This model fitting tool incorporates cyclic coordinate descent and
    majorization-minimization approaches to fit a variety of regression models
    found in large-scale observational healthcare data.  Implementations focus
    on computational optimization and fine-scale parallelization to yield
    efficient inference in massive datasets.  Please see:
    Suchard, Simpson, Zorych, Ryan and Madigan (2013) <doi:10.1145/2414416.2414791>.
License: Apache License 2.0
LazyData: Yes
Biarch: true
URL: https://github.com/ohdsi/cyclops
BugReports: https://github.com/ohdsi/cyclops/issues
Depends: R (>= 3.5.0)
Imports: rlang, Matrix, Rcpp (>= 0.12.12), Andromeda, dplyr, methods,
        survival, tidyr, bit, bit64
LinkingTo: Rcpp, RcppEigen (>= 0.3.2)
Suggests: testthat, readr, MASS, gnm, ggplot2, microbenchmark, cmprsk,
        boot, rsample
NeedsCompilation: yes
RoxygenNote: 7.3.2
Encoding: UTF-8
Packaged: 2026-05-26 15:24:17 UTC; msuchard
Author: Marc A. Suchard [aut, cre],
  Martijn J. Schuemie [aut],
  Trevor R. Shaddox [aut],
  Yuxi Tian [aut],
  Jianxiao Yang [aut],
  Eric Kawaguchi [aut],
  Sushil Mittal [ctb],
  Observational Health Data Sciences and Informatics [cph],
  Marcus Geelnard [cph, ctb] (provided the TinyThread library),
  Rutgers University [cph, ctb] (provided the HParSearch routine),
  R Development Core Team [cph, ctb] (provided the ZeroIn routine)
Maintainer: Marc A. Suchard <msuchard@ucla.edu>
Repository: CRAN
Date/Publication: 2026-05-27 06:20:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-27 08:28:06 UTC; unix
Archs: Cyclops.so.dSYM
