% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Gpu.R
\name{setOpenCLDevice}
\alias{setOpenCLDevice}
\title{Set GPU device}
\usage{
setOpenCLDevice(name)
}
\arguments{
\item{name}{String: Name of GPU device}
}
\description{
\code{setOpenCLDevice} set GPU device
}
