% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{listGPUDevices}
\alias{listGPUDevices}
\title{List available GPU devices}
\usage{
listGPUDevices()
}
\description{
\code{listGPUDevices} list available GPU devices
}
